#!/bin/sh

### HELP
# 1. make deb-pkg (options):  <file: scripts/package/builddeb>
# 2. localversion handling:   <file: scripts/setlocalversion>
# 3. KBUILD_USER_* variables: <file: scripts/mkcompile_h>
# 4. Prereqs/Preparation:     <http://kernel-handbook.alioth.debian.org/ch-common-tasks.html#s4.2.1>

### PREREQS: Install build-essential, fakeroot and dpkg-dev packages (optional: git)!
#apt-get update
#apt-get install build-essential fakeroot dpkg-dev
#apt-get install git

### Language settings
export LANG=C
export LC_ALL=C

### Base and Linux-Next source directory
BASE_DIR=$(pwd)
SRC_DIR="${BASE_DIR}/linux-next"

### Clone Linux-Next remote GIT tree
#GIT_URL="git://git.kernel.org/pub/scm/linux/kernel/git/next/linux-next.git"
#git clone ${GIT_URL}

### Change to build directory
cd ${SRC_DIR}

### Make and compiler settings
MAKE="make"
MAKE_JOBS=$(getconf _NPROCESSORS_ONLN)
CC_FOR_BUILD="gcc-4.6"

### Uploader: realname/nickname and email settings
export DEBFULLNAME="Sedat Dilek"
export DEBEMAIL="sedat.dilek@gmail.com"
uploader="dileks"

### Upstream-version settings
version=$(awk '/^VERSION = / {print $3}' Makefile)
patchlevel=$(awk '/^PATCHLEVEL = / {print $3}' Makefile)
sublevel=$(awk '/^SUBLEVEL = / {print $3}' Makefile)
extraversion=$(awk '/^EXTRAVERSION = / {print $3}' Makefile)
upstream_version="${version}.${patchlevel}.${sublevel}${extraversion}"

### Debian-revision settings
# NOTE-1: $revision will be appended to $debian_revision and $mylocalversion
# NOTE-2: Upstream: $revision starts at "1" with each new v3.x.y (next: y+1) or v3.x.0-rcX (next: X+1)
# NOTE-3: Linux-Next: $revision starts at "1" with each linux-next release
# EXAMPLE: debian_revision="1+dileks1"
revision="2"
uploader_revision="1"
debian_revision="${revision}+${uploader}${uploader_revision}"

### Extract $mylocalversion_next
# Note: $mylocalversion_next will be appended to $mylocalversion
# EXAMPLE: mylocalversion_next="next20130125"
mylocalversion_next=$(git tag | grep ^'next-' | tail -1 | tr -d '-')

### My featureset settings
# Here: Use own "iniza" featureset
myfeatureset="iniza"

### My kernel-flavour settings
# Here: Use own "small" kernel-flavour (localmodconfig-ed plus some debug-options enabled) 
mykernelflavour="small"

### Customized LOCALVERSION settings
# NOTE-1: $mylocalversion is attached as suffix to "include/generated/utsrelease.h" file
# NOTE-2: Usage of LOCALVERSION=$mylocalversion suppresses "+" suffix
# NOTE-3: See also "#define UTS_RELEASE" in utsrelease.h file.
# EXAMPLE-1: "next20130125" (mylocalversion_next) + "1" (revision) + "iniza" (myfeatureset) + "small" (mykernelflavour)
# EXAMPLE-2: mylocalversion="-next20130125-1-iniza-small"
mylocalversion="-${mylocalversion_next}-${revision}-${myfeatureset}-${mykernelflavour}"

### My kernel-release (aka KVER)
mykernelrelease="${upstream_version}${mylocalversion}"

### Build-log file
BUILD_LOG_FILE="build-log_${mykernelrelease}.txt"

### make options
MAKE_OPTS="CC=${CC_FOR_BUILD} -j${MAKE_JOBS} KBUILD_BUILD_USER=${DEBEMAIL} LOCALVERSION=${mylocalversion}"

### deb-pkg options
# NOTE-1: Change '-' in $upstream_version to '~' caused by $extraversion (3.8.0-rc4 VS. 3.8.0~rc4)
# NOTE-2: Add $myfeatureset to $debian_revision
# EXAMPLE: KDEB_PKGVERSION="3.8.0~rc4~next20130125-1~iniza+dileks1"
upstream_version=$(echo $upstream_version | tr '-' '~')
debian_revision="${revision}~${myfeatureset}+${uploader}${uploader_revision}"
DEB_PKG_OPTS="KDEB_PKGVERSION=${upstream_version}~${mylocalversion_next}-${debian_revision}"

echo "make options ...... ${MAKE_OPTS}"
echo "dep-pkg options ... ${DEB_PKG_OPTS}"

### Remove any existing localversion* file(s) as $LOCALVERSION is used
# Note: Linux-Next ships localversion-next file.
rm -v -f localversion*

### Start build
fakeroot ${MAKE} ${MAKE_OPTS} deb-pkg ${DEB_PKG_OPTS} 2>&1 | tee ../${BUILD_LOG_FILE}
